// Ce code n'est pas fonctionnel, et n'est pas destiné à être compilé.
// Il s'agit simplement d'extraits commentés du programme de l'EFIS AvionicsDuino,
// destinés à expliquer le fonctionnement du système de menus.


// ******************************************* Déclaration d'un typedef struct, puis d'un tableau d'éléments de cette structure, définissant l'arborescence des menus ********************************************************
/*
   Voir le schéma du système de menus : https://i.imgur.com/oD18FKH.jpg.
   Définitions :
   - Option de menu : tout texte pouvant être sélectionné avec l'encodeur rotatif, puis validé avec le bouton de cet encodeur, en vue d'agir sur le système, ou d'ouvrir une autre option.
   - Niveaux de menus : le menu principal est le niveau 1, chaque option du niveau 1 peut elle même ouvrir un sous-menu de niveau 2 qui lui-même...etc.
   - Numéro d'option : chaque option des menus et sous-menus se voit affecter un numéro unique avec la logique suivante : la première option du niveau 1 a le numéro 10, et on ajoute une unité pour les numéros des options suivantes
   du même niveau. Donc les numéros des options du niveau 1 sont : 10, 11, 12, 13...etc sans dépasser 19. Chaque option peut ouvrir un sous menu de niveau 2, dont le premier numéro est obtenu en multipliant par 10 le numéro 
   de l'option "mère". On rajoute des unités pour les sous-options suivantes. Par exemple l'option n° 12 de niveau 1 peut ouvrir les sous-options filles de numéros 120, 121, 122...etc. de niveau 2, sans dépasser 129. 
   On multiplie encore par 10 pour le niveau suivant.
   A chaque niveau, le numéro se terminant par zéro est affecté à l'option qui permet de retourner au niveau précédent (Back), simplement en divisant son numéro par 10. La dernière option de chaque sous-menu permet 
   de quitter complètement le menu (Quit). On ne peut pas, avec des nombres entiers, avoir plus de 10 options par menu/sous-menu. Mais pour une application simple, c'est suffisant. 
   Pour éviter d'avoir plus de 10 options, qui de plus ne pourraient pas s'afficher en bandeau horizontal en bas de l'écran, il faut créer des sous-menus.
   - Index : les options sont stockées dans un tableau où chaque option est repérable par son index

   La structure des options de menus et l'algorithme de traitement de ces options ont été conçus de telle façon qu'il soit très simple de faire évoluer ces menus, de modifier, de supprimer ou de rajouter des options.
   Voir le schéma de l'algorithme : https://i.imgur.com/eA0QNdb.jpg
   */
     //Le tableau va jouer le rôle d'une table de base de données pour contenir tous les éléments (options) des menus/sous-menus.
     // Chaque ligne du tableau est l'équivalent d'un enregistrement (ou entrée) de cette table. Chaque variable de la structure est un champ de cette table.
typedef struct     
{
  char* label;              // Libellé de l'option sous forme d'une C string. On évite d'utiliser un objet String.
  uint32_t numero;          // Numéro permettant de l'identifier, il est unique (équivalent d'une clé primaire)
  char action;              // Type d'action attendue si cette option est validée : Q->Quitte, S->Sous-menu à ouvrir, B->Back retour option mère, P->Procédure d'ajustement d'une valeur, A->Autre Action
  uint8_t nbOptionsSoeurs;  // Nombre d'options soeurs, c'est à dire toutes les options d'un même niveau de menu/sous-menu. Ce champ est calculé et initialisé pendant le setup.
  uint8_t indexOfOm;        // Index dans le tableau menu (APRES tri) de l'option fille (si action='S') ou de l'option mère (si action = 'B'). Champ calculé et initialisé pendant le setup. Voir le rôle des index : https://i.imgur.com/c1LsPR6.jpg
}OptionMenu;                // Nom de la structure (toujours à la fin en cas de déclaration de typedef struct)


// Déclaration d'un tableau sans dimension, nommé "menu", de type "OptionMenu". Chaque ligne du tableau caractérise une option de menu/sous-menu.
// Initialisation des trois premiers champs, "label", "numero" et "action", de toutes les lignes de ce tableau. Les 2 autres champs seront initialisés dans le setup.
// La présentation indentée ci-dessous, triée pour bien exposer l'arborescence des menus, permet une gestion facile des options de menus (modification, ajout ou suppression d'options).
// Mais avant utilisation, ce tableau sera trié différement dans le setup, par ordre croissant du champ "numero". Ce nouveau tri permettra l'exploitation des menus par l'algorithme.
// Le setup montre les résultats avant et après tri sur la console de l'IDE Arduino. 

OptionMenu menu[]=                              // Les champs "numéro" d'un même menu/sous-menu doivent OBLIGATOIREMENT se suivre toutes les unités, sans aucun trou.
{
  {"FLIGHT",10,'S'},                              
           {" BACK",  100,'B'},
           {"  QNH",  101,'P'},
           {"  ------", 102,'A'},
           {"ResetG", 103,'A'},
           {"  QUIT", 104,'Q'},
  
  {"   GEN",11,'S'},

           {" BACK",  110,'B'},
           {"  Uvit",  111,'S'},

                       {" BACK", 1110,'B'},
                       {" Km/h", 1111,'A'},
                       {"Noeuds",1112,'A'},
                       {"  QUIT",1113,'Q'},
           
           {"MgDev", 112,'P'},
           {"  Time", 113,'S'},

                       {" BACK", 1130,'B'},
                       {" Year", 1131,'P'},
                       {"Month", 1132,'P'},
                       {" Day",  1133,'P'},
                       {"Hours", 1134,'P'},
                       {" Min",  1135,'P'},
                       {" Sec",  1136,'P'},
                       {"Valid", 1137,'A'},
                       {"  QUIT",1138,'Q'},

           {"PrsCor", 114,'P'},
           {"  Lum",  115,'P'},
           {" DatLog",116,'S'},

                       {" BACK",  1160,'B'},
                       {"   Del", 1161,'A'},
                       {"  Start",1162,'A'},
                       {"  Stop", 1163,'A'},
                       {" Export",1164,'A'},
                       {"  QUIT", 1165,'Q'},
           
           {"  QUIT", 117,'Q'},

  {"Timers",12,'S'}, 

           {" BACK",  120,'B'},
           {"Depart", 121,'A'},
           {"Chrono", 122,'A'},
           {" Down",  123,'P'},
           {"  QUIT", 124,'Q'},   
  
  {"  QUIT",13,'Q'},   
  
  {" ",1000000,'\0'} // Marqueur de fin de tableau, permettant d'en compter les éléments
};

//******************************************************************** Variables Encodeur ***********************************************************************
int ValeurPositionEncodeur; // Dès que l'encodeur est mis en rotation, cette variable évolue
int ValeurPositionEncodeurOld = 0; // Ancienne position de l'encodeur pour la navigation dans les menus
int ValeurPositionEncodeurOldSetVal = 0; // Ancienne position de l'encodeur lors de l'ajustage d'une valeur de variable
bool Bouton_Clic; // Flag indiquant que le bouton vient d'être pressé


#define pinA 35
#define pinB 37
#define pinSW 36 // Bouton encodeur sur pin 36
volatile int8_t encoderPos = 0;
int8_t lastPos = 0;
int8_t sens = 0;
boolean A_set = false;
boolean B_set = false;

//********************************************************************* Variables Menus **************************************************************************
bool Menu_Ouvert = false; // Flag indiquant que le menu est ouvert
bool SetVal = false; // Flag indiquant qu'une valeur de paramètre est en cours de modification
uint8_t indexOptionMenuEnCours = 0;   // Indique l'index (dans le tableau) de l'option de menu en cours   
int16_t * ptrGen = NULL; // Un pointeur générique de type entier qui servira à pointer sur l'une des valeurs entières ci-dessous, quand il s'agira de la régler dans une option de menu.
                         // Toutes les valeurs à régler ainsi doivent être du type int16_t.
uint16_t abscisseMenu=0, ordonneeMenu = 254; // Coordonnées du coin supérieur gauche du menu
uint8_t hauteurCase = 18, largeurCase = 53; // Taille des cases des options

void setup() {
// **************************************************************************************** Initialisation et tri des options des menus  **********************************************************************************************************************
// Pour faciliter la gestion des options de menu, ces options sont affichées pendant le setup sur le terminal série, avant et après tri.
  uint8_t nb = NbOptionsMenu(); // Calcul du nombre de lignes du tableau
  Serial.print ("Nombre de lignes utiles du tableau : ");Serial.println(nb); Serial.println(); // Par lignes "utiles" on désigne celle qui contiennent les options des menus, donc à l'exclusion de la ligne marqueur de fin
  Serial.println("**********Tableau avant tri et initialisation des index***********");
  Serial.println("Index------Label--------------Numero--Action---NbOptS--IndexOfOm--");
  for (byte n=1; n<=nb+1; n++)
  {
    sprintf(buf,"%2d",(n-1));
    Serial.print(buf); Serial.print(char(9)); 
    sprintf(buf,"%-20s",(menu[n-1].label));
    Serial.print (buf); Serial.print(char(9)); 
    sprintf (buf,"%4d",(menu[n-1].numero));
    Serial.print (buf); Serial.print (char(9));
    Serial.print(menu[n-1].action); Serial.print (char(9)); 
    sprintf(buf,"%2d",(menu[n-1].nbOptionsSoeurs));
    Serial.print(buf); Serial.print(char(9));       
    sprintf(buf,"%2d",(menu[n-1].indexOfOm));
    Serial.print(buf); Serial.println();   
  }
  triBulleTableau(); // on trie le tableau dans l'ordre du champ "numero"
  initTable();       // Initialisation des champs "nbOptionsSoeurs" et indexOfOm"
  
  Serial.println();
  Serial.println("**********Tableau après tri et initialisation des index***********");
  Serial.println("Index------Label--------------Numero--Action---NbOptS--IndexOfOm--");  
  for (byte n=1; n<=nb+1; n++)
  {
    indexOptionMenuEnCours = n-1;
    sprintf(buf,"%2d",(n-1));
    Serial.print(buf); Serial.print(char(9)); 
    sprintf(buf,"%-20s",(menu[n-1].label));
    Serial.print (buf); Serial.print(char(9)); 
    sprintf (buf,"%4d",(menu[n-1].numero));
    Serial.print (buf); Serial.print (char(9));
    Serial.print(menu[n-1].action); Serial.print (char(9)); 
    sprintf(buf,"%2d",(menu[n-1].nbOptionsSoeurs));
    Serial.print(buf); Serial.print(char(9));       
    sprintf(buf,"%2d",(menu[n-1].indexOfOm));
    Serial.println(buf); 
  }
  tft.println("Menus                 : OK");
  delay(28);

}

void loop() {

//  ***************************************************************************** Traitement d'un clic sur le bouton de l'encodeur *******************************************************************************************

  if (Bouton_Clic == true) // Le bouton peut avoir été pressé pour trois raisons : soit ouvrir le menu s'il était fermé, et si le menu était ouvert, soit ouvrir un sous-menu, soit valider la modification d'un paramètre.
  {
    if (Menu_Ouvert == false) // si le menu était fermé il faut juste l'ouvrir et attendre la suite
    { 
      Menu_Ouvert = true; // Le menu est maintenant déclaré ouvert 
      OuvreMenu(0); // Ouverture du menu sur le premier index du tableau, donc pour afficher le menu principal.   
    }
    else  // on va traiter ici tous les cas où le bouton a été pressé pour soit valider une option de menu      (Bouton_Clic == true, ET Menu_Ouvert==true, ET SetVal == false)
          //                                                               soit valider une valeur de paramètre (Bouton_Clic == true, ET Menu_Ouvert==true, ET SetVal == true)
    {
      if (SetVal == false)
       {
        //Quatre cas différents à envisager :
           // Q) Soit on est sur l'option Quit d'un menu quelconque -> alors on ferme simplement le menu
           // S) Soit on est sur une option qui demande l'ouverture d'un sous menu -> alors on ouvre ce sous-menu
           // B) Soit on est sur une option Back d'un menu quelconque -> alors il faut remonter au menu parent et l'afficher en lieu et place du menu actuel
           // P) Soit on est sur une option quelconque d'ajustement d'une valeur numérique -> on ajuste cette valeur
           // A) Soit enfin une autre action est à faire
         
         switch (menu[indexOptionMenuEnCours].action)
          {
            case 'Q':                                                                                                               // Cas Q) : On ferme la fenêtre Menu
               Menu_Ouvert = false;
               FermeMenu (); // Effacement de la fenêtre du menu.
               break;
    
            case 'S':                                                                                                               // Cas S) : On passe au sous menu fils du menu/sous-menu actuel
               OuvreMenu(menu[indexOptionMenuEnCours].indexOfOm);
               break;
    
            case 'B':                                                                                                                // Cas B) : On revient au menu parent, sur son option "Back" ou "Quit"
               OuvreMenu(menu[indexOptionMenuEnCours].indexOfOm);;
               break;
    
            case 'P':                                                                                                                 // Cas P) : On va procéder au réglage d'une valeur numérique
              SetVal = true; 
              encoderPos=0; // On met l'encodeur à zéro
              //A_set = digitalRead(pinA);
              //B_set = digitalRead(pinB);
              ValeurPositionEncodeurOld = ValeurPositionEncodeur;
              ValeurPositionEncodeurOldSetVal = 0; // et aussi la valeur de l'ancienne position affectée au réglage d'une valeur
              Bouton_Clic = false; // On commence par mettre ce flag sur false, car la validation de la donnée saisie (en clicquant sur le bouton) le remettra sur True
              switch (menu[indexOptionMenuEnCours].numero)
                {                                                  // Si la valeur à ajuster doit, après validation, être enregistrée dans l'EEPROM, on indique ici l'adresse EEPROM, ou 1000 si valeur non concernée
                  case 101  : ptrGen = &QNH;                       OffsetEFISeepromAdresse =  0;   break; 
                  case 112  : ptrGen = &magDev;                    OffsetEFISeepromAdresse =  2;   break;
                  case 114  : ptrGen = &correctionPression;        OffsetEFISeepromAdresse = 10;   break;
                  case 115  : ptrGen = &luminosite;                OffsetEFISeepromAdresse = 12;   break;                              
                  case 123  : ptrGen = &dureeMinuteur;             OffsetEFISeepromAdresse = 1000; flag_minuteur = false; tft.setCursor (320,222);tft.print ("     "); break;
                  case 1131 : ptrGen = &YY;                        OffsetEFISeepromAdresse =  8;   break;
                  case 1132 : ptrGen = &MM;                        OffsetEFISeepromAdresse = 1000; break;
                  case 1133 : ptrGen = &DD;                        OffsetEFISeepromAdresse = 1000; break;
                  case 1134 : ptrGen = &HH;                        OffsetEFISeepromAdresse = 1000; break;
                  case 1135 : ptrGen = &Mn;                        OffsetEFISeepromAdresse = 1000; break;
                  case 1136 : ptrGen = &Sec;                       OffsetEFISeepromAdresse = 1000; break;
                  default   : break;
                }
              tft.fillRect((((menu[indexOptionMenuEnCours].numero)%10)*largeurCase), (ordonneeMenu-hauteurCase+3),largeurCase, hauteurCase-3, RA8875_RED);
              tft.setCursor(((menu[indexOptionMenuEnCours].numero)%10)*largeurCase +(largeurCase/4), ordonneeMenu+4-hauteurCase);
              tft.setTextColor(RA8875_WHITE, RA8875_RED);
              tft.setFont(Arial_10);
              tft.print(*ptrGen); 
              tft.setFont(Arial_11);
              break; // break du case 'P'
    
            case 'A' :                                                                                                                 // Cas A) : On va procéder à une autre action
              switch (menu[indexOptionMenuEnCours].numero)
                {
                  case 102  : OuvreMenu(indexRetour(indexOptionMenuEnCours)); break;
                  case 103  : accGmax = 1.0; accGmin = 1.0; EEPROM.put(EFISeepromAdresse + 14,accGmax); EEPROM.put(EFISeepromAdresse + 18,accGmin); OuvreMenu(indexRetour(indexOptionMenuEnCours)); break; // Remise à "zéro" des accélarations max et min

                  case 112  : OuvreMenu(indexRetour(indexOptionMenuEnCours)); break;
                   
                  case 121  : if (flag_chrono1==false) {t_chrono1=now();flag_chrono1=true;}            // Si le chrono1 "départ" ne tourne pas, on le démarre
                              else {tft.setCursor(80,222); tft.print("        "); flag_chrono1=false;} // S'il tourne, on l'efface et on l'arrête 
                              break;

                  case 122  : if (flag_chrono2==false) {t_chrono2=now();flag_chrono2=true;}              // Si le chrono2 ne tourne pas, on le démarre
                              else {tft.setCursor(220,222); tft.print("        "); flag_chrono2=false;}  // S'il tourne, on l'efface et on l'arrête
                              break;

                  case 1111 : facteurConversionVitesse = 1.0; OuvreMenu(indexRetour(indexOptionMenuEnCours));   break;   // Vitesses affichées en Km/h
                  case 1112 : facteurConversionVitesse = 1.852; OuvreMenu(indexRetour(indexOptionMenuEnCours)); break;   // Vitesses affichées en noeuds           

                  case 1137 : setTime(HH,Mn,Sec,DD,MM,YY); break;    // Validation d'une heure réglée manuellement (en cas de pile du SRTC en panne...) 

                  case 1161 : if (SD.exists("log.txt")){SD.remove("log.txt");fichier = SD.open("log.txt");fichier.close();} flagDataLogging=false; break; // Efface toutes les données du fichier log (en fait supprime puis recrée un fichier vide)
                  case 1162 : fichier = SD.open("log.txt", FILE_WRITE); flagDataLogging=true; break; // Ouvre le fichier log pour pouvoir commencer à y ajouter des données
                  case 1163 : fichier.close(); flagDataLogging=false; break; // Arrête d'ajouter des données
                  case 1164 : fichier.close(); flagDataLogging=false; fichier = SD.open("log.txt");if (fichier) {while (fichier.available()) {Serial.write(fichier.read());}fichier.close();}break; // Exporte le fichier des données en serial USB
                } 
              break; // break du case 'A'
              
          }
       } // fin de if (SetVal == false)
       
       else // donc début de if (SetVal == true) (routine de traitement d'un clic APRES modification d'une valeur)
       {
         if (OffsetEFISeepromAdresse != 1000) EEPROM.put(EFISeepromAdresse + OffsetEFISeepromAdresse, *ptrGen); // si la variable est concernée, on enregistre sa valeur modifiée et validée dans l'EEPROM
         if (dureeMinuteur <= 0){flag_minuteur = false; tft.setCursor (320,222);tft.print ("     ");}
         else if (flag_minuteur ==false){t_minuteur = now()+ dureeMinuteur*60; flag_minuteur = true;}
         SetVal = false; // La modification d'une valeur a été validée par le clic, la procédure de modification est terminée
         ptrGen = NULL;  // Le pointeur n'a plus de raison de pointer sur quoique ce soit      
         OuvreMenu(indexRetour(indexOptionMenuEnCours)); // et on revient à la première option du menu en cours
       }
    }// fin du premier else
    Bouton_Clic = false; // Le traitement de cette pression sur le bouton est terminé 
  }

// ************************************************************************* Polling de la position de l'encodeur à chaque passage dans Loop ************************************************************************************
  if (Menu_Ouvert == true) // On ne s'occupe de l'encodeur que si le menu est ouvert
  {
    if (SetVal == false) // si setVal==false, la rotation de l'encodeur dirige la navigation dans les options du menu en cours
     {
        //ValeurPositionEncodeur = encodeur.read() / 4; // On lit la position actuelle de l'encodeur à chaque passage dans Loop 
        ValeurPositionEncodeur = encoderPos;
        tft.setTextColor (RA8875_WHITE, RA8875_PINK);
        if (ValeurPositionEncodeur != ValeurPositionEncodeurOld) // Si l'encodeur a "bougé" depuis le dernier passage dans Loop, alors on va traiter ce mouvement
         { 
           // On commence par fixer les limites mini et maxi entre lesquelles on peut faire bouger le curseur. Ces limites dépendent des index mini et maxi du menu dans lequel on se trouve
           uint16_t mini = menu[indexOptionMenuEnCours].numero; // mini contient le numéro de l'option en cours
           mini = mini%10; // mini contient maintenant l'offset de l'index de l'option en cours par rapport à l'index de l'option la plus basse du menu/sous-menu en cours. Les numéros d'options doivent se suivre, sans trou !!
           mini= indexOptionMenuEnCours - mini; // et mini contient maintenant l'index de l'option la plus basse du menu/sous-menu en cours
           uint8_t maxi = mini + menu[mini].nbOptionsSoeurs -1; //maxi contient l'index de l'option la plus haute du menu/sous-menu en cours
           if ((ValeurPositionEncodeur > maxi-mini) || (ValeurPositionEncodeur < 0)) 
             {
               encoderPos=0; // On limite les mouvements du curseur entre mini et maxi, avec retour automatique au mini si le maxi est dépassé 
               ValeurPositionEncodeur = encoderPos;
             }
           indexOptionMenuEnCours = ValeurPositionEncodeur + mini; //On calcule la nouvelle valeur de la variable indexOptionMenuEnCours               
           tft.drawRect ((ValeurPositionEncodeurOld * largeurCase), ordonneeMenu,largeurCase,hauteurCase, BckgrndMenus);   // et on efface la position précédente de ce cadre     
           tft.drawRect ((ValeurPositionEncodeur * largeurCase), ordonneeMenu,largeurCase,hauteurCase, RA8875_WHITE);            //On dessine le cadre blanc qui entoure l'option de menu active
         }
        ValeurPositionEncodeurOld = ValeurPositionEncodeur; // on enregistre la position actuelle de l'encodeur pour comparaison au prochain passage dans Loop
     }
     else // si SetVal == true, la rotation de l'encodeur permet d'ajuster une valeur numérique
     {
    //ValeurPositionEncodeur = encodeur.read() / 4; // On lit la position actuelle de l'encodeur à chaque passage dans la boucle loop
    ValeurPositionEncodeur = encoderPos;
    if (ValeurPositionEncodeur != ValeurPositionEncodeurOldSetVal) // Si l'encodeur a bougé depuis le dernier passage dans la boucle, alors on va traiter ce mouvement
      {
        *ptrGen = *ptrGen + sens; // on ajuste la valeur de la variable pointée en fonction du sens de rotation de l'encodeur
        verificationValeurs();
        tft.fillRect((((menu[indexOptionMenuEnCours].numero)%10)*largeurCase), (ordonneeMenu-hauteurCase+3),largeurCase, hauteurCase-3, RA8875_RED);
        tft.setCursor(((menu[indexOptionMenuEnCours].numero)%10)*largeurCase +(largeurCase/4), ordonneeMenu+4-hauteurCase);
        tft.setTextColor(RA8875_WHITE, RA8875_RED);
        tft.setFont(Arial_10);
        tft.print(*ptrGen);
        tft.setFont(Arial_11);        
      }
    ValeurPositionEncodeurOldSetVal = ValeurPositionEncodeur; // on enregistre la position actuelle de l'encodeur pour comparaison au prochain passage dans la boucle
     }
  }


}


//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//                                                                                          Fonctions liées à l'encodeur rotatif et aux menus 
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void verificationValeurs() // Cette fonction vérifie si certains paramètres modifiés dans les menus restent dans les limites autorisées
{
  if (luminosite>16) luminosite = 16;       if (luminosite<1) luminosite = 1;
  if (dureeMinuteur>60) dureeMinuteur = 60; if (dureeMinuteur<0) dureeMinuteur = 0;
}

// *********************************************************************************** Routines d'interruptions liées à l'encodeur *******************************************************************************

void pressSW() 
{
  Bouton_Clic = true;
}

void changeA() 
{
  A_set = digitalRead(pinA) == HIGH;
  sens = (A_set != B_set) ? +1 : -1;
  encoderPos += sens;
}

void changeB() 
{
  B_set = digitalRead(pinB) == HIGH;
  sens = (A_set == B_set) ? +1 : -1;
  encoderPos += sens;

}

// ***************************************************************************************** Procédure d'ouverture du menu ******************************************************************************************************

void OuvreMenu (uint8_t ndx) // Cette fonction reçoit l'index de la première option du menu/sous-menu à ouvrir
{
  indexOptionMenuEnCours = ndx; // La valeur de la variable globale indexOptionMenuEnCours est ajustée en fonction de la valeur reçue par la fonction
  tft.fillRect (abscisseMenu, ordonneeMenu-hauteurCase+2, tft.width()-abscisseMenu, hauteurCase*2-2, BckgrndMenus); // Affichage du fond coloré de la zone du menu, on efface donc ce qui était affiché précédemment
  tft.drawLine(abscisseMenu+1, ordonneeMenu-hauteurCase+2, tft.width()-abscisseMenu, ordonneeMenu-hauteurCase+2, RA8875_WHITE);
  encoderPos=0; // On positionne l'encodeur à 0
  //A_set = digitalRead(pinA);
  //B_set = digitalRead(pinB);
  ValeurPositionEncodeur = 0; // ValeurPositionEncodeur à 0.
  ValeurPositionEncodeurOld = 0; // Et l'ancienne position à 0 également.
  tft.setFont(Arial_11); // Une police pour des menus bien lisibles
  tft.setTextColor (RA8875_WHITE, BckgrndMenus);
  tft.setCursor(abscisseMenu+1, ordonneeMenu+5-hauteurCase); // On positionne le curseur à l'emplacement du titre du menu
  if(indexOptionMenuEnCours==0)
  {
    tft.print("MENU GENERAL");
  }
  else
  {
    int8_t i=0;
    tft.setTextColor (RA8875_WHITE, BckgrndMenus);
    while(menu[indexOptionMenuEnCours+i].numero != (menu[indexOptionMenuEnCours].numero)/10) i--; // recherche de l'index (et donc du nom à afficher) de l'option mère
    tft.print(menu[indexOptionMenuEnCours+i].label);
  }
      for (byte n = ndx; n < ndx + menu[ndx].nbOptionsSoeurs; n++)  // puis on affiche les options du menu
      {
        tft.setCursor ((largeurCase*(n-ndx))+2, ordonneeMenu+3);
        tft.print(menu[n].label);
      }  
   tft.drawRect ((ValeurPositionEncodeur*largeurCase),ordonneeMenu, largeurCase,hauteurCase, 0xFFFF); //On dessine le cadre blanc qui entoure la première option de menu.
}
// ********************************************************************************************* Procédure de fermeture du menu **************************************************************************************************
void FermeMenu()
{
  tft.fillRect (abscisseMenu, ordonneeMenu-hauteurCase, tft.width()-abscisseMenu, hauteurCase*2, RA8875_PINK); // Affichage d'un fond rose transparent à l'emplacement de la fenêtre du menu.
  Horizon.redessine(); // Pour réafficher la bille correctement
  tft.setFontDefault(); 
  tft.setFontScale (0);
}
// ************************************************ Cette fonction retourne, à partir d'un index donné quelconque, l'index de la première option soeur (numéro terminant par un zéro) **********************************************
uint8_t indexRetour(uint8_t indexQuelconque)
{
  int8_t i = 0;
  while(menu[indexQuelconque+i].numero > ((menu[indexQuelconque].numero)/10)*10) i--;
  return (indexQuelconque+i);
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//                                                                                Fonctions utilisées lors de l'initialisation des options des menus
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void initTable() // Cette fonction calcule et initialise les champs "nbOptionsSoeurs" et "indexOfOm" de chaque enregistrement, lors du setup.
{
  for (uint8_t i=0; i<NbOptionsMenu(); i++)
  {
    uint16_t n = menu[i].numero;
    int8_t j = 0;
    if ((n/10)*10 == n) // Si division entière par 10 puis multiplication par 10 ne change pas le numéro, c'est que ce numéro se termine par un zéro, donc une première option de menu --> calcul du nombre d'options soeurs
    {
       while (menu[i+j].numero < n+10) j++; // on compte d'abord les options suivantes (triées par numéros croissants) dont le numéro est dans les 9 suivants : à la fin de la boucle while, j est le nombre d'options soeurs.
       for (uint8_t k = 0; k<j; k++) menu[i+k].nbOptionsSoeurs = j; // et avec une deuxième boucle, on affecte le nomnbre d'options soeurs aux options concernées.
       j = 0;
    }
    
    if(menu[i].action == 'S') // si on est sur une option permettant d'ouvrir une option fille, il faut calculer l'index de la première option fille
    {
       while(menu[i+j].numero != n*10) j++; // le numéro de la première option fille est le numéro de l'option mère multiplié par 10
       menu[i].indexOfOm = i+j;
       j=0;
    }
    
    if(menu[i].action == 'B')
    {
       while(menu[i+j].numero != (n/100)*10) j--; // le numéro de la première option mère est le numéro de l'option fille divisé par 10, en ôtant les unités. Obtenu par division entière par 100 puis multiplication par 10)
       menu[i].indexOfOm = i+j;
    }
  }
}

void triBulleTableau() // tri bulle tout à fait clasique : comparer les éléments adjacents et les échanger s'ils sont dans le mauvais ordre, 
                       // puis recommencer sur tout le tableau jusqu'à ce que tous les éléments adjacents soient dans le bon ordre.
                       // Pour un tableau de n éléments, on réalise n-1 comparaisons à la première "passe",  à l'issue de laquelle l'élément le plus grand a été "poussé" à sa place.
                       // on réalise donc n-2 comparaisons à la 2ème passe, et le 2ème plus grand élément est poussé à sa place. Et ainsi de suite.
                       // Après n-1 passes, tous les éléments sont triés.
                       // Ce n'est pas, et de loin, l'algorithme le plus efficace en terme de nombre total de comparaisons. Mais c'est le plus simple à coder. Et il ne sert qu'une seule fois, durant le setup.

{
  char * texte; uint32_t nombre; char lettre; // 3 variables intermédiaires pour procéder aux échanges entre lignes du tableau
  for (byte i = NbOptionsMenu()-1; i>0; i--)
  {
    for (byte j = 1; j <= i; j++)
    {
      if (menu[j-1].numero > menu[j].numero) // si le champ "numero" de l'enregistrement j-1 est supérieur à celui de l'enregistrement j, alors on va permuter toutes les valeurs entre ces deux enregistrements
      {
        texte =   menu[j].label;             // On stocke provisoirement les valeurs de l'enregistrement j dans des variables intermédiaires
        nombre = menu[j].numero;
        lettre =  menu[j].action;

        menu[j].label        = menu[j-1].label;        // on attribue les valeurs de l'enregistrement j-1 à l'enregistrement j
        menu[j].numero       = menu[j-1].numero;
        menu[j].action       = menu[j-1].action;

        menu[j-1].label        = texte;
        menu[j-1].numero       = nombre;
        menu[j-1].action       = lettre;
      }
    }
  }
}

uint8_t NbOptionsMenu()
{
  uint8_t i=0;
  while (menu[i].numero != 1000000) { i=i+1;}
  return i; // Ce nombre ne prend pas en compte la dernière ligne du tableau (marqueur de fin). Les index "utiles" du tableau vont donc de 0 à NbOptionsMenu()-1. (NbOptionsMenu() donne l'index de la ligne marqueur de fin).
}
